/*
 * KeyPresses.java
 *
 * Created on 12 May 2009, 09:53
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package dosagenumbers;
import java.util.*;

/**
 *
 * @author pcairns
 */
public class KeyPresses {
    
    private final static int NUM_PRESSES = 4;
    private final static String[] fKeys = {"0","1","2","3","4","5","6","7","8","9",".","X"};
    private enum State {STARTED, HAVEPOINT, ENTERED};
    private final static Double INVALID = new Double( -1 );
    private final static double OVERALL_ERROR_RATE = 0.22;
    
    Vector fPresses;
    Vector fValues;
    Vector fProbs;
    Vector fValid;
    PatternMatcher fMatcher;
    
    private String fCorrectDosage;

    
    /** Creates a new instance of KeyPresses */
    public KeyPresses() {
        fMatcher = new PatternMatcher();
        
        fPresses = new Vector();
        fValues = new Vector();
        fValid = new Vector();
        initialisePresses();
    }
    
    private void initialisePresses(){
        int lTotalCombo = (int) Math.pow(fKeys.length,NUM_PRESSES);
        int i,j;
        int[] lIndexes = new int[NUM_PRESSES];
        String lThisSeq;
        StringBuffer lThisSeqBuffer;
        Double lThisValue;
        Boolean lIsValid;
        boolean lIncrement;
        int lThisIndex;
        
        for( j =0; j<NUM_PRESSES; j++){ 
            lIndexes[j] = 0;
        }
        
        for( i = 0; i < lTotalCombo; i++){
           lThisSeqBuffer = new StringBuffer();
           for( j =0; j<NUM_PRESSES; j++){
               lThisSeqBuffer.append( fKeys[ lIndexes[ j] ]);
           }
           lThisSeq = lThisSeqBuffer.toString();
           lThisValue = interpretSeq( lThisSeq );
           lIsValid = validateSeq( lThisSeq );
           fPresses.add( lThisSeq );
           fValues.add(lThisValue );
           fValid.add( lIsValid );
           
           //System.out.println( lThisSeq + ": " + lThisValue );
           
           lIncrement = true;
           for( j = NUM_PRESSES -1; j>=0 && lIncrement; j--){
               lThisIndex = lIndexes[ j];
               lThisIndex++;
               if( lThisIndex < fKeys.length){
                   lIndexes[j] = lThisIndex;
                   lIncrement = false;
               }
               else{
                   lIndexes[j] = 0;
                   lIncrement = true;
               }
           }//for j (incrementing indices)        
        }//for i
    }
    
    /*
     *Pulled out a separate interpreter so that evaluation rules are explicit.
     */
    private Double interpretSeq( String pSeq ){
        int i;
        String lThisChar;
        StringBuffer lNewValue = new StringBuffer();
        State lState = State.STARTED;
        Double lAnswer;
        
        for( i=0; i< NUM_PRESSES && lState != State.ENTERED; i++ ){
            lThisChar = pSeq.substring( i, i+1 );
            if( lThisChar.equals("X") ){//stop at a termination
                lState = State.ENTERED;
            } 
            else if( lThisChar.equals(".")){//Ignore double pts
               if( lState != State.HAVEPOINT){
                   lState = State.HAVEPOINT;
                   lNewValue.append( lThisChar );
               }
            }
            else{
                lNewValue.append( lThisChar );
            }    
        }
        
        //Deal with exceptional cases
        if( lNewValue.toString().equals(".") ){
            lAnswer = 0.0;
        }
        else if( lNewValue.toString().equals("") ){
            lAnswer = 0.0;
        }
        else{
            lAnswer = new Double( lNewValue.toString() );
        }
        
        return lAnswer;        
    }//interpretSeq
    
    private Boolean validateSeq( String pSeq ){
        Boolean lAnswer;
        boolean lIsValid = false;
        
        int lMatchIndex = fMatcher.matchDosage( pSeq );
        if( lMatchIndex != fMatcher.UNMATCHED ){
            lIsValid = true;
        }
        lAnswer = new Boolean( lIsValid );
        
        return lAnswer;
    }
    
   
    public void setDosage( String pDosage ){
        String lThisChar, lThatChar;
        String lThisKeyPress;
        int lErrorCount = 0;
        int i,j;
        int lNumOfPresses = fPresses.size();
        double lProb;
        Boolean lIsValid;
        double lRunningProb = 0;
        double lErrorRate = OVERALL_ERROR_RATE/11.0;
        
        if( pDosage.length() != NUM_PRESSES ){
            System.out.println("Dosage not right length: " + pDosage);
        }
        else{  
            fProbs = new Vector();
            fCorrectDosage = pDosage;
            for( i= 0; i < lNumOfPresses; i++ ){
                lThisKeyPress = (String) fPresses.elementAt( i );
                
                lErrorCount = 0;
                for( j = 0; j<NUM_PRESSES; j++){
                    lThisChar = fCorrectDosage.substring(j, j+1);
                    lThatChar = lThisKeyPress.substring(j,j+1);
                    if( !lThisChar.equals(lThatChar) ){
                        lErrorCount++;
                    }
                }
                lProb = Math.pow( lErrorRate,  lErrorCount) * Math.pow( 1-OVERALL_ERROR_RATE, NUM_PRESSES-lErrorCount );
                fProbs.add( new Double( lProb ));
                
                //lRunningProb += lProb;
            }
        }    
    }
    
    public double sumValidDosageProbs(){
        int i;
        int lNumOfPresses = fPresses.size();
        double lTotalProb = 0.0;
        double lThisProb;
        boolean lIsValid;
        
        for( i = 0; i < lNumOfPresses; i ++){
            lThisProb = ((Double) fProbs.elementAt( i )).doubleValue();
            lIsValid = ((Boolean) fValid.elementAt(i) ).booleanValue();
            if (lIsValid) lTotalProb += lThisProb;
        }
        
        return lTotalProb;
    }
    
    
    
    public void out(){
        int i;
        int lNumOfPresses = fPresses.size();
        String lThisSeq;
        Double lThisValue;
        Double lThisProb;
        Boolean lIsValid;
        
        for( i = 0; i< lNumOfPresses; i++){
            lThisSeq = (String) fPresses.elementAt( i );
            lThisValue = (Double) fValues.elementAt( i );
            lThisProb = (Double) fProbs.elementAt( i );
            lIsValid = (Boolean) fValid.elementAt( i );
            System.out.println( lThisSeq + ": " + lThisValue + ": " + lThisProb + ": " + lIsValid );
        }
    }
}
