/*
 * KeySequences.java
 *
 * Created on 18 May 2009, 14:06
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package dosagenumbers;
import java.util.*;

/**
 *
 * @author pcairns
 */
public class KeySequences {
    
    public final static double TOLERANCE = 0.000001;
    
    private int fLength;
    private Vector fSequences;
    private String fCorrectDosage;
    
    /** Creates a new instance of KeySequences */
    public KeySequences(int pLength ) {
        fLength = pLength;
        fSequences = new Vector();
        
        initialiseSequences();
    }
    
    private void initialiseSequences(){
        int lTotalCombo = (int) Math.pow(Keys.numKeys(), fLength);
        int i,j;
        int[] lIndexes = new int[ fLength ];
        Sequence lThisSeq;
        StringBuffer lThisSeqBuffer;
        boolean lIncrement;
        int lThisIndex;
        
        for( j =0; j< fLength; j++){ 
            lIndexes[j] = 0;
        }
        
        for( i = 0; i < lTotalCombo; i++){
           lThisSeqBuffer = new StringBuffer();
           for( j =0; j < fLength; j++){
               lThisSeqBuffer.append( Keys.getKey( lIndexes[j] ));
           }
           lThisSeq = new Sequence( lThisSeqBuffer.toString() );
           fSequences.add( lThisSeq );
           
           //System.out.println( lThisSeq + ": " + lThisValue );
           
           lIncrement = true;
           for( j = fLength -1; j>=0 && lIncrement; j--){
               lThisIndex = lIndexes[ j];
               lThisIndex++;
               if( lThisIndex < Keys.numKeys() ){
                   lIndexes[j] = lThisIndex;
                   lIncrement = false;
               }
               else{
                   lIndexes[j] = 0;
                   lIncrement = true;
               }
           }//for j (incrementing indices)        
        }//for i
    }//initialiseSequences

    public void setDosage( String pDosage ){
        String lThisChar, lThatChar;
        Sequence lThisSeq;
        int lErrorCount = 0;
        int i,j;
        int lNumOfSeqs = fSequences.size();
        double lProb;
        double lRunningProb;
        double lCheckProb = 0;
       
        if( pDosage.length() == fLength ){
            fCorrectDosage = pDosage;
            for( i= 0; i < lNumOfSeqs; i++ ){
                lThisSeq  = (Sequence) fSequences.elementAt( i );
                
                lRunningProb = 1;
                for( j = 0; j < fLength; j++){
                    lThisChar = fCorrectDosage.substring(j, j+1);
                    lThatChar = lThisSeq.getSequence().substring(j,j+1);
                    
                    if( lThisChar.equals(lThatChar) ){
                        lRunningProb *= (1 - ErrorRates.TOTAL);
                    }
                    else if( lThatChar.equals(Keys.TRUNC)){
                        lRunningProb *= ErrorRates.TRUNCATE;
                    }
                    else{
                        lRunningProb *= ErrorRates.SPECIFIC;
                    }
                }//for j
                
                lThisSeq.setProb( lRunningProb );
                //lCheckProb += lRunningProb;
            }//for i
        } else{
            System.out.println("Dosage not right length: " + pDosage);
            
        }
        
        //System.out.println( fCorrectDosage + ", " + lCheckProb );
    }

    public ErrorSummary summariseProbs( double pBadLimit ){
        double lCorrect = 0;
        double lBad = 0;
        double lInvalid = 0;
        double lBadAndInvalid = 0;
        double lThisProb;
        double lThisValue;
        double lCorrectValue = new Sequence( fCorrectDosage ).getValue();
        double lValueRatio;
        double lLowerLimit = 1/pBadLimit;
        ErrorSummary lSummary = new ErrorSummary( fCorrectDosage );
        boolean lIsBad;
        
        int i;
        int lNumSeqs = fSequences.size();
        Sequence lThisSeq;
        
        for( i = 0; i < lNumSeqs; i++){
            lThisSeq = (Sequence) fSequences.elementAt( i );
            lThisProb = lThisSeq.getProb();
            lThisValue = lThisSeq.getValue();
            lValueRatio = lThisValue/lCorrectValue;
            
            lIsBad = false;
            if( lValueRatio > pBadLimit || lValueRatio < lLowerLimit ){
                lIsBad = true;
                lBad += lThisProb;
            }
            
            if( !lThisSeq.isValid() ){
                lInvalid += lThisProb;
                if( lIsBad ) lBadAndInvalid += lThisProb;
            }
         
            if( lThisValue < lCorrectValue + TOLERANCE && lThisValue > lCorrectValue - TOLERANCE ){
                lCorrect += lThisProb;
            }
            
            lSummary.setCorrectProb( lCorrect );
            lSummary.setBadDoseProb( lBad );
            lSummary.setInvalidProb( lInvalid );
            lSummary.setBadAndInvalidProb( lBadAndInvalid );  
            lSummary.setBadR( pBadLimit );
        }
        
        return lSummary;
    }
    
    public StringList asStringList(){
        StringList lAnswer = new StringList();
        int i;
        int lNumOfPresses = fSequences.size();
        Sequence lThisSeq;
        
        for( i = 0; i< lNumOfPresses; i++){
            lThisSeq = (Sequence) fSequences.elementAt( i );
            lAnswer.add( lThisSeq.asString() );
        }
        
        return lAnswer;
    }
    
     public void out(){
        int i;
        int lNumOfPresses = fSequences.size();
        Sequence lThisSeq;
        
        for( i = 0; i< lNumOfPresses; i++){
            lThisSeq = (Sequence) fSequences.elementAt( i );
            System.out.println( lThisSeq.asString() );
        }
    }

}
