/*
 * Sequence.java
 *
 * Created on 18 May 2009, 14:08
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package dosagenumbers;

/**
 *
 * @author pcairns
 */
public class Sequence {
    
    //Interpreter. Could probably pull this out in future for different types of interpretation
    private enum State {STARTED, HAVEPOINT, ENTERED};

    
    private String fSequence;
    private double fValue;//Value sequence interprets to, can have different interpreters in future
    private double fProb; //Prob of sequence in comparison to some dosage
    private double fPropR; //Proportion of this value to some dosage
    private boolean fIsValid;// Is syntactically a valid number
    private boolean fIsValidTrunc; //If contains truncation, truncation is valid
    private DosageValidator fValidator;
    
    /** Creates a new instance of Sequence */
    public Sequence( String pSeq) {
        fSequence = pSeq;
        fValidator = new DosageValidator();
        calculateValue();
        validateSeq();
    }
    
    private void calculateValue(){
        int i;
        int lSeqLength = fSequence.length();
        String lThisChar;
        StringBuffer lNewValue = new StringBuffer();
        State lState = State.STARTED;
        double lAnswer;
        
        for( i=0; i< lSeqLength && lState != State.ENTERED; i++ ){
            lThisChar = fSequence.substring( i, i+1 );
            if( lThisChar.equals( Keys.TRUNC ) ){//stop at a termination
                lState = State.ENTERED;
            } 
            else if( lThisChar.equals( Keys.POINT )){//Ignore double pts
               if( lState != State.HAVEPOINT){
                   lState = State.HAVEPOINT;
                   lNewValue.append( lThisChar );
               }
            }
            else{
                lNewValue.append( lThisChar );
            }// if lThisChar   
        }//for i
        
        //Deal with exceptional cases
        if( lNewValue.toString().equals( Keys.POINT ) ){
            lAnswer = 0.0;
        }
        else if( lNewValue.toString().equals( "" ) ){
            lAnswer = 0.0;
        }
        else{
            lAnswer = Double.valueOf( lNewValue.toString() );
        }
        
        fValue = lAnswer;      
    }
    
    private void validateSeq(){
       int lValidation;
       int lTruncPos = fSequence.indexOf( Keys.TRUNC );
       String lTruncation;
       
       fIsValid = false;
       lValidation = fValidator.matchDosage( fSequence );
       if( lValidation != fValidator.UNMATCHED ){
           fIsValid = true;
       }
       
       if( lTruncPos == -1 ){
           fIsValidTrunc = fIsValid;
       }
       else{
           fIsValidTrunc = false;
           lTruncation = fSequence.substring( 0, lTruncPos );
           lValidation = fValidator.matchDosage( lTruncation );
           
           if( lValidation != fValidator.UNMATCHED ){
               fIsValidTrunc = true;
           }
       }  
    }
    
    public void setProb( double pProb ){
        fProb = pProb;
    }
    
    public void setPropR( double pPropR ){
        fPropR = pPropR;
    }
    
    public double getValue(){
        return fValue;
    }
    
    public String getSequence(){
        return fSequence;
    }
    
    public boolean isValid(){
        return fIsValidTrunc;
    }
    
    /*public boolean isValidTrunc(){
        return fIsValidTrunc;
    }
     */
    
    public double getProb(){
        return fProb;
    }
    
    public double getPropR(){
        return fPropR;
    }
    
    public String asString(){
        String lAnswer = fSequence + ", " + fValue + ", " + fIsValid + ", " + fIsValidTrunc + ", " + fProb + ", " + fPropR;
        return lAnswer;
    }
    
    public void out(){
        System.out.println( this.asString() );
    }
    
}
